from platform import platform
import ktc
import onnx
import sys
import cv2
from PIL import Image
import numpy as np
from sklearn.utils.extmath import softmax

# Variables for workflow processing
model_path = '/docker_mount/Models/cracks_CNN_2_1.tflite_nosmax'
images_path = '/docker_mount/Images/'
onnx_path1 =  '/docker_mount/onnx/cracks_CNN_2_1_onnx1_nosmax.onnx'
onnx_path2 =  '/docker_mount/onnx/cracks_CNN_2_1_onnx2_nosmax.onnx'
input_shape = [1, 224, 224, 4]
input_names = ["img"]
input_name = "img"


def preprocess(img_file_path):
    image = cv2.imread(img_file_path)
    # resize image to match model input size (this case: width 224  height 224)
    image = cv2.cvtColor(image, cv2.COLOR_RGB2RGBA)
    image = cv2.resize(image , (224, 224), interpolation=cv2.INTER_LINEAR)
    np_data = image/255
    return np_data

def postProcess(output_Arr):
    CATEGORIES = ["No Cracks", "Cracks"]
    print(output_Arr)
    prediction = softmax(output_Arr[0])
    print(prediction)
    prediction = np.argmax(prediction, axis=1)   
    print(prediction)
    print("result:", CATEGORIES[prediction[0]]) 


# ## TF Lite to ONNX
# result_m = ktc.onnx_optimizer.tflite2onnx_flow(model_path)
# onnx.save(result_m, onnx_path1)

## ONNX Optimization
onnx_file = onnx.load(onnx_path2)
m = ktc.onnx_optimizer.onnx2onnx_flow(onnx_file, eliminate_tail=True, opt_matmul=True)


# save the optimized onnx model
onnx.save(m, onnx_path2)

## Section 3
km = ktc.ModelConfig(32773, "0001", "520", onnx_path=onnx_path2)
eval_result = km.evaluate()
print("\nNpu performance evaluation result:\n" + str(eval_result))



input_data = [preprocess("/docker_mount/Images/2.jpg")]
inf_results = ktc.kneron_inference(input_data, onnx_file=onnx_path2, input_names=input_names)
print('Section 3 E2E simulator result:')
postProcess(inf_results)

## Section 4
# Preprocess images and create the input mapping
input_images = [
    preprocess(images_path + "2.jpg"),
    preprocess(images_path + "4.jpeg"),
    preprocess(images_path + "7.jpg"),
    preprocess(images_path + "10.jpeg"),
    preprocess(images_path + "5.jpg"),
    preprocess(images_path + "16.jpeg"),
    preprocess(images_path + "21.jpg"),
    preprocess(images_path + "23.jpeg"),
]
input_mapping = {input_name: input_images}

# Quantization
bie_path = km.analysis(input_mapping, output_bie = None, threads = 4)

# E2E simulator (fixed point)
input_data = np.array(input_data)
# radix = ktc.get_radix(input_images)
fixed_results = ktc.kneron_inference(input_data, bie_file=bie_path, platform=520, input_names=input_names)
print('Section 4 E2E simulator result:')
postProcess(fixed_results)

## Section 5
# Batch compile
compile_result = ktc.compile([km])
print(compile_result)

# E2E simulator (hardware)
hw_results = ktc.kneron_inference(input_data, nef_file=compile_result, platform=520)
print('Section 5 E2E simulator result:')
print(hw_results)

